setClass("segment", representation(lrr="numeric", baf="numeric", gtype="character"))
setClass("info", representation(chr="numeric" , l="numeric", w="numeric", x="numeric", bfreq="numeric", h="character", start="numeric", end="numeric"))
	
cnagen <- function(lengths=lapply(1:6,function(x)c(100)), times = rep(1,6), weights2=c(0.66), weights3=c(0.66), copynumbers = list(list(c(1)),list(c(1,3)),list(c(1,3,4))), # CNA parameters
		replicates = 1, purities = c(1,0.5), distSNP = 1, varDistSNP = FALSE, # sample parameters
		noiseBAF = 0.03, extremeBAF = 0.3, reduceExtrBAF = TRUE, pfbMode = "array", # BAF parameters
		noiseLRR = 0.2, constLRRNoise = TRUE, lrrLevels = c(-.75,-.5,0,0.29,.5,.66,.79,.9,1), autoBase = TRUE, baseMean = 0, baseSD = 0.1, waveSD = 0.02, # LRR parametrs
		annotationDir = "data", lrrFirstDir = "data_lrrbaf", bafFirstDir = "data_baflrr", # output parameters
		scaffold = NULL, waveGCSD = 0.03, intratHet = 0 # real data parameters (scaffold is a data frame with columns [Chr, Len, LOH, C1, BA1])
	) {
	
	piece = function(l=100, w=1, x=2, bfreq=NULL, loh=c("no","germline","somatic")[1], bn=0.03, ln=0.2, pfbs=rep(0.5,100), constP = 0.3,
					redBafExtrs=FALSE, lrrLevels=c(-.75,-.5,0,0.29,.5,.66,.79,.9,1), constLrrNoise=FALSE) { 

		# lrrLevels = lrrLevels * q # Compress signal by the compression factor (deprecated)
		r = lrrLevels[x+1] # Expected LRR levels for normal cells and tumour subclones copy numbers
		if (!constLrrNoise) {
			lrrNoise = c(10,1.4,1,0.9,0.85,0.8,0.8) * ln
			moreLrrNoise = rep(0.16, length(lrrLevels)-length(lrrNoise)) * ln
			lrrNoise = c(lrrNoise, moreLrrNoise)
		} else {
			lrrNoise = c(2, rep(0.2, length(lrrLevels)-1)) * ln
		}
		g = lrrNoise[x+1] # LRR levels for normal cells and tumour subclones copy numbers
		bafNoise = bn # SD of Gaussian noise for BAF
		if (redBafExtrs) rr = 0.5 else rr = 1
		
		# if (length(x)>2) minx = 0 # any allelic imbalance for complex CNAs
		# else minx = 1 # only heterozygous option for simple CNAs
		# status = as.vector(unlist(lapply(x, function(xx) sample(minx:ceiling(xx/2))[1])))

		bafNoiseComp = rnorm(l,sd=bafNoise)
		bafSignalComp = matrix(nrow=length(x), ncol=l)
		if (loh=="germline")
			bafSignalComp[1,] = rbinom(l,1,pfbs)*2 # If the piece has germline (not acquired) homozygosity
		else
			bafSignalComp[1,] = rbinom(l,x[1],pfbs) # The normal (CN2) cells without LOH
		gtype = bafSignalComp[1,]
		
		if (length(x) > 1)
			for (i in 2:length(x)) # for each tumour clone
				if (x[i] == 0) { # copy number 0 for corresponding subclone
					bafSignalComp[i,] = rnorm(l,mean=0.5,sd=0.33) # a truncated Gaussian distributed between 0 and 1
					bafSignalComp[i,] = as.vector(unlist(lapply(bafSignalComp[i,], function(x) max(0,min(1,x)))))
				} else {
					for (j in 1:l) # for each SNP
						if (bafSignalComp[1,j]==2) {
							bafSignalComp[i,j] = x[i] # if normal is homozygous, tumour too
						} else if (bafSignalComp[1,j]==0) {
							bafSignalComp[i,j] = 0 # if normal is homozygous, tumour too
						} else { # bafSignalComp[1,j]==1
							if (loh!="no") { # If the piece has either germline or acquired homozygosity
								bafSignalComp[i,j] = rbinom(1,1,0.5)*x[i] # LOH, only 0 or x[i] values
							} else {
							# Heterozygosity in germline (and thus, also tumour).
							# Alteration can have any allelic (im)balance if intra-tumour complexity;
							# otherwise, it is heterozygous (homozygous regions are defined separately in cnagen input)
								# if (runif(1)>0.5)
									bafSignalComp[i,j] = bfreq[i]
								# else
									# bafSignalComp[i,j] = x[i] - bfreq[i]
							}
						}
				}
				
		for (j in 1:dim(bafSignalComp)[2])
			if (runif(1)>0.5) bafSignalComp[,j] = x - bafSignalComp[,j]

		if (length(which(x==0)) > 0) { # there is a subclone with cn0
			if (w[1]==0.0 && length(x==2)) { # a pure tumour sample with cn0
				bafSignalComp = bafSignalComp[2,] # the signal is just the generated truncated Gaussian
			} else { # some signal exists, so disregard cn0 subclone, as it will not affect the baf signal
				if (is.null(dim(bafSignalComp[-which(x==0),]))) # colSums won't operate on single-row object
					bafSignalComp = (w[-which(x==0)] * bafSignalComp[-which(x==0),]) / sum(w * x)
				else
					bafSignalComp = colSums(w[-which(x==0)] * bafSignalComp[-which(x==0),]) / sum(w * x)
			}
		} else { # trivial case, there is not a subclone with cn0
			bafSignalComp = colSums(w * bafSignalComp) / sum(w * x)
		}
		if (length(which(bafSignalComp==0))>0)
		bafNoiseComp[which(bafSignalComp==0)] = as.vector(unlist(lapply(bafNoiseComp[which(bafSignalComp==0)], function(x) if (runif(1)>(1-constP)) 0 else abs(x)*rr)))
		if (length(which(bafSignalComp==1))>0)
		bafNoiseComp[which(bafSignalComp==1)] = as.vector(unlist(lapply(bafNoiseComp[which(bafSignalComp==1)], function(x) if (runif(1)>(1-constP)) 0 else -abs(x)*rr)))
		baf = bafSignalComp+bafNoiseComp
		# transform values close to extremes into extremes
		# baf = as.vector(unlist(lapply(baf, function(x) if(x>0.9){x=max(1,x+0.01)} else if(x<0.1){x=min(0,x-0.01)} else{x})))
		# avoid out of range values
		baf = as.vector(unlist(lapply(baf, function(x) if(x>1){x=max(0,2-x)} else if(x<0){x=min(1,abs(x))} else{x})))

		if (length(which(x==0)) > 0) { # there is a subclone with cn0
			if (w[1]==0.0 && length(x==2)) { # a pure tumour sample with cn0
				lrrSignalComp = -rchisq(l,8)/16+lrrLevels[1]+rnorm(l,sd=0.5) # the signal is just a low long tailed distribution
				lrr = lrrSignalComp # the signal is already noise, so no need to add more
			} else { # some signal exists, so disregard cn0 subclone, as it will not affect the lrr signal
				lrrSignalComp = lapply(r[-which(x==0)], function(x) rep(x,l))
				lrrNoiseComp = lapply(g[-which(x==0)], function(g) rnorm(l,sd=g))
				lrr = do.call(rbind,lrrNoiseComp) + do.call(rbind,lrrSignalComp)
				wN = w[-which(x==0)] / sum(w[-which(x==0)]) # reweight weights proportions excluding cn0 weight, so that they still sum 1
				lrr = lrr * wN
				if (! is.null(dim(lrr))) # colSums won't operate on single-row object
					lrr = colSums(lrr)
			}
		} else { # trivial case, there is not a subclone with cn0
			lrrSignalComp = lapply(r, function(x) rep(x,l))
			lrrNoiseComp = lapply(g, function(g) rnorm(l,sd=g))
			lrr = do.call(rbind,lrrNoiseComp) + do.call(rbind,lrrSignalComp)
			lrr = lrr * w
			lrr = colSums(lrr)
		}
		
		gtype = as.vector(unlist(lapply(gtype, function(x) if (x==0) "AA" else if (x==1) "AB" else "BB")))
		
		segment = new("segment", lrr=lrr, baf=baf, gtype = gtype)
	}

	getPfb <- function(pfb,l) {
		# cummulative population B allele frequency (PFB) quantiles derived from SNPs in DPK-cologenetics arrays
		pfbReal = c(0.018,0.023,0.027,0.03,0.032,0.035,0.037,0.039,0.041,0.044,0.051,0.057,0.063,0.07,0.075,0.082,
		0.087,0.093,0.099,0.104,0.11,0.115,0.12,0.126,0.131,0.136,0.141,0.147,0.152,0.157,0.162,0.167,0.171,0.177,
		0.181,0.186,0.191,0.196,0.201,0.205,0.21,0.215,0.22,0.224,0.229,0.233,0.238,0.243,0.247,0.252,0.256,0.261,
		0.265,0.27,0.274,0.279,0.283,0.288,0.292,0.296,0.301,0.305,0.309,0.313,0.318,0.322,0.326,0.331,0.335,0.339,
		0.344,0.348,0.352,0.356,0.36,0.365,0.369,0.373,0.378,0.382,0.386,0.39,0.395,0.399,0.403,0.407,0.411,0.416,
		0.42,0.424,0.429,0.433,0.437,0.442,0.446,0.45,0.455,0.459,0.464,0.469,0.474,0.478,0.483,0.488,0.492,0.497,
		0.501,0.505,0.51,0.514,0.519,0.523,0.527,0.532,0.536,0.541,0.545,0.549,0.554,0.558,0.563,0.567,0.572,0.576,
		0.581,0.585,0.59,0.594,0.599,0.603,0.608,0.613,0.617,0.622,0.627,0.632,0.636,0.641,0.646,0.651,0.656,0.661,
		0.666,0.671,0.676,0.681,0.686,0.691,0.696,0.701,0.707,0.712,0.717,0.722,0.728,0.733,0.738,0.744,0.749,0.755,0.761,
		0.766,0.772,0.778,0.783,0.789,0.795,0.801,0.807,0.813,0.819,0.825,0.831,0.838,0.844,0.851,0.857,0.864,0.871,0.877,
		0.885,0.891,0.898,0.905,0.912,0.92,0.926,0.935,0.94,0.948,0.951,0.954,0.957,0.959,0.962,0.965,0.968,0.974,0.982,1)
		pfbSimple = c(0,0.5,1)

		if (pfb=="array") {
			pfbs = sample(pfbReal,replace=TRUE,size=l)
		} else if (pfb=="threepeaks") {
			pfbs = sample(pfbSimple,replace=TRUE,size=l)
		} else if (pfb=="uniform") {
			pfbs = runif(l)
		} else if (pfb=="maxinfo") {
			pfbs = rep(0.5, l)
		} else { # pfb = a value
			pfbs = rep(pfb, l)
			toswitch = sample(1:l, size=ceiling(l*runif(1)))
			pfbs[toswitch] = 1 - pfb
		}
		pfbs
	}

	readOriginalCalls = function(info) {
		table = do.call(rbind,lapply(info, function(x) c(x@chr,x@start,x@end,paste(x@w,collapse=", "),paste(x@x,collapse=", "),x@h,x@l)))
		colnames(table) = c("chr","start","end","proportions","copy.numbers","homozygosity","length")
		table = as.data.frame(table)
		table$main.cn = as.vector(unlist(lapply(info, function(x) if(length(x@x)==1) x@x[1] else x@x[1+which(x@w[2:length(x@w)]==max(x@w[2:length(x@w)]))])))
		table$main.cn.prop = as.vector(unlist(lapply(info, function(x) if(length(x@w)==1) x@w[1] else max(x@w[2:length(x@w)]))))
		table = transform(table, chr=as.numeric(as.character(chr)))
		table = transform(table, start=as.numeric(as.character(start)))
		table = transform(table, end=as.numeric(as.character(end)))
		# table = table[sort.list(table$start),]
		table = table[with(table, order(chr, start)), ]
		table
	}
	
	hybrid = !is.null(scaffold)

	if (!hybrid) {
		if (length(lengths)!=6 || length(times)!=6)
			stop("There are 6 region types. Please enter the set of lengths and the repetion times for each of them.")
		if (any(weights2<=0) || any(weights3<=0) || any(weights2>=1) || any(weights3>=1))
			stop("Subclone proportions should be in the open interval (0,1).")
		if (length(copynumbers)!=3)
			stop("Parameter 'copynumbers' should have length 3: copy numbers for simple, and copy number sets for 2-subclone and 3-subclone alterations.\n
			Please use NULL within list() if necessary.")
		if (any(unlist(copynumbers)<0))
			stop("Copy numbers should be equal or greater than zero.")
	}
		
	if (replicates<1)
		stop("You should at least generate one sample.")
	if (length(purities)<1)
		stop("You should select at least one purity level.")
	if (any(purities<0) || any(purities>1))
		stop("Purities should be in the range between zero and one.")
	if (! is.finite(distSNP))
		stop("Distance between SNPs should be a finite positive number.")
	if (distSNP < 1)
		stop("Distance between SNPs should be a finite positive number.")
	if (any(extremeBAF<0) || any(extremeBAF>1))
		stop("Extreme values of homozygous SNPs should account for a proportion of between zero and one.")
	if (! pfbMode %in% c("array","threepeaks","uniform","maxinfo"))
		if (pfbMode<0 || pfbMode>1)
			stop("Select a PFB sampling mode from between 'array','threepeaks','uniform', 'maxinfo' and a value between zero and one.")
		
	if (noiseBAF<0 || noiseLRR<0)
		stop("The noise factors for LRR and BAF signals should be equal or greater than zero.")
	if (length(lrrLevels)+1 < max(unlist(copynumbers)))
		stop("Please specify LRR values at least up to the highest copy number that you are using.")
	if (baseSD<0 || waveSD<0)
		stop("Standard deviations should be equal or greater than zero.")

	# if (any(unlist(copynumbers)==2))
		# warning("Alterations with copy number 2 are not really alterations...")
	

	if (!hybrid) {
		l0 = lengths[[1]]
		l1 = lengths[[2]]
		lsloh = lengths[[3]]
		lgloh = lengths[[4]]
		l2 = lengths[[5]]
		l3 = lengths[[6]]
		x1 = copynumbers[[1]]
		x2 = copynumbers[[2]]
		x3 = copynumbers[[3]]
		x1 = lapply(x1,function(i) c(2,i))
		x2 = lapply(x2,function(i) c(2,i))
		x3 = lapply(x3,function(i) c(2,i))
		timesNormal = times[[1]]
		times1Clone = times[[2]]
		timesSLOH = times[[3]]
		timesGLOH = times[[4]]
		times2Clones = times[[5]]
		times3Clones = times[[6]]
	}	

	# directories
	dir.create(annotationDir, showWarnings=FALSE)
	dir.create(lrrFirstDir, showWarnings=FALSE)
	dir.create(bafFirstDir, showWarnings=FALSE)
	
	isFirst = TRUE
	n = replicates # number of replicates
	
	for (purity in purities)
	for (d in 1:n) {
		baf = lrr = c()
		infos = list()

		if (hybrid) {
			meta = paste("p",purity,"_n",d,sep="")
		
			totalLength = sum(scaffold$Len)
			if (isFirst)
				pfb = getPfb(pfbMode,totalLength)
		
			for (i in 1:dim(scaffold)[1]) {
				if (scaffold$LOH[i]==1) { # germline LOH
					h = "germline"
				} else { # somatic LOH or heterozygosity
					if (scaffold$BA1[i]%in%c(0,scaffold$CN1[i])) # somatic LOH
						h = "somatic"
					else # heterozygosity
						h = "no"
				}
				if (i==1) st = 1
				else st = st + scaffold$Len[i-1]
				en = st + scaffold$Len[i] - 1
				if (intratHet>0) {
					hetProp = runif(1)*intratHet
					w = c(1-purity,purity*(1-hetProp), purity*hetProp)
					if (scaffold$CN1[i] %in% c(0,1))
						hetCN = scaffold$CN1[i] + 1
					else if (scaffold$CN1[i] %in% c(8))
						hetCN = scaffold$CN1[i] - 1
					else
						hetCN = ifelse(runif(1)>0.5, scaffold$CN1[i]-1, scaffold$CN1[i]+1)
					x = c(2,scaffold$CN1[i], hetCN)
					info = new("info", l=scaffold$Len[i],w=w,x=x,bfreq=c(scaffold$BA1[i], sample(0:ceiling(hetCN/2))[1]),
							h=h,start=st,end=en,chr=scaffold$Chr[i])
				} else {
					info = new("info", l=scaffold$Len[i],w=c(1-purity,purity),x=c(2,scaffold$CN1[i]),bfreq=scaffold$BA1[i],
							h=h,start=st,end=en,chr=scaffold$Chr[i])
				}
				infos = c(infos, info)
				aPiece = piece(info@l,info@w,info@x,info@bfreq,info@h,
				noiseBAF,noiseLRR,pfb[st:en],extremeBAF,reduceExtrBAF,lrrLevels,constLRRNoise)
				baf = c(baf,aPiece@baf)
				lrr = c(lrr,aPiece@lrr)
			}
		} else {
			nc = 1 - purity
			w_1 = w_2 = w_3 = list()
			w_1[[1]] = c(nc,1-nc) # a single clone w/ x% contamination
			if (length(weights2)>0)
			for (i in 1:length(weights2)) {
				w_2[[i*2-1]] = c(nc,(1-nc)*weights2[i],(1-nc)*(1-weights2[i]))
				w_2[[i*2]] = c(nc,(1-nc)*(1-weights2[i]),(1-nc)*weights2[i])
			}
			if (length(weights3)>0)
			for (i in 1:length(weights3)) {
				minClone = (1-weights3[i])/2
				w_3[[i*3-2]] = c(nc,(1-nc)*weights3[i],(1-nc)*minClone,(1-nc)*minClone)
				w_3[[i*3-1]] = c(nc,(1-nc)*minClone,(1-nc)*weights3[i],(1-nc)*minClone)
				w_3[[i*3]] = c(nc,(1-nc)*minClone,(1-nc)*minClone,(1-nc)*weights3[i])
			}
			co =list(w_1,w_2,w_3)

			clones = c()
			if (length(co[[2]])>0 && times2Clones>0 && length(l2)>0 && length(x2)>0)	
				clones = c(clones,2)
			if (length(co[[3]])>0 && times3Clones>0 && length(l3)>0 && length(x3)>0)
				clones = c(clones,3)
			
			if (! is.null(clones))
				meta = paste("p",purity,"_c",paste(clones,collapse="-"),"_n",d,sep="")
			else
				meta = paste("p",purity,"_n",d,sep="")
			# metainfoString = paste("p",purity,"_c",paste(clones,collapse="-"),"_n",d,"_b",signif(baselevel,2),sep="")
			cat(paste("Generating ",meta,"... ",sep=""))
			
			if (timesNormal>0 && length(l0)>0)
			for (rr0 in 1:timesNormal)
			for (i in 1:length(l0)) {
				infos = c(infos, new("info", chr=1,l=l0[i],w=1,x=2,h="no"))
			}
			if (times1Clone>0 && length(l1)>0 && length(x1)>0)
			for (i in 1:length(l1)) 
				for (k in 1:length(x1))
					for (rr1 in 1:times1Clone)
						if (k!=2) {
							h = ifelse(x1[[k]][2]==1, "somatic", "no")
							infos = c(infos, new("info", chr=1,l=l1[i],w=co[[1]][[1]],x=x1[[k]],h=h))
						}
			if (timesSLOH>0 && length(lsloh)>0 && length(x1)>0)
			for (rrloh in 1:timesSLOH)
			for (i in 1:length(lsloh)) 
				for (k in 1:length(x1)) {
					infos = c(infos, new("info", chr=1,l=lsloh[i],w=co[[1]][[1]],x=x1[[k]],h="somatic"))
				}
			if (timesGLOH>0 && length(lgloh)>0 && length(x1)>0)
			for (rrloh in 1:timesGLOH)
			for (i in 1:length(lgloh)) 
				for (k in 1:length(x1)) {
					infos = c(infos, new("info", chr=1,l=lgloh[i],w=co[[1]][[1]],x=x1[[k]],h="germline"))
				}	
			if (length(co[[2]])>0 && times2Clones>0 && length(l2)>0 && length(x2)>0) {
				for (rr2 in 1:times2Clones)
				for (i in 1:length(l2))
					for (k in 1:length(x2))
						for (j in 1:length(co[[2]])) {
							infos = c(infos, new("info", chr=1,l=l2[i],w=co[[2]][[j]],x=x2[[k]],h="no"))
						}	
			} 
			if (length(co[[3]])>0 && times3Clones>0 && length(l3)>0 && length(x3)>0) {
				for (rr3 in 1:times3Clones)
				for (i in 1:length(l3))
					for (k in 1:length(x3))
						for (j in 1:length(co[[3]])) {
							infos = c(infos, new("info", chr=1,l=l3[i],w=co[[3]][[j]],x=x3[[k]],h="no"))
						}
			}
			
			totalLength = sum(unlist(lapply(infos,function(x) x@l)))
			if (isFirst)
				pfb = getPfb(pfbMode,totalLength)
		
			rand = sample(length(infos))
			xx = vector("list", length(infos)*2)
			st = 1
			en = infos[[rand[1]]]@l
			infos[[rand[1]]]@start = st
			infos[[rand[1]]]@end = en
			if (length(infos[[rand[1]]]@x)>2) minx = 0 # any allelic imbalance for complex CNAs
			else minx = 1 # only heterozygous option for simple CNAs
			status = as.vector(unlist(lapply(infos[[rand[1]]]@x, function(xx) sample(minx:ceiling(xx/2),1))))
			aPiece = piece(infos[[rand[1]]]@l,infos[[rand[1]]]@w,infos[[rand[1]]]@x,bfreq=status,infos[[rand[1]]]@h,
				noiseBAF,noiseLRR,pfb[st:en],extremeBAF,reduceExtrBAF,lrrLevels,constLRRNoise)
			baf = c(baf,aPiece@baf)
			lrr = c(lrr,aPiece@lrr)
			xx[[1]] = infos[[rand[1]]]@x
			pos = c()
			pos[1] = rand[1]
			isSimilar =	function(i,p) (paste(xx[[i]],collapse=",")==paste(infos[[p]]@x,collapse=","))
			for (p in rand[2:length(rand)]) {
				for (i in 1:(length(infos)*2)) {
					if ( !isSimilar(i,p) && is.null(xx[[i+1]])) {
						xx[[i+1]] = infos[[p]]@x
						pos[i+1] = p
						break
					}
				}
			}
			for (ppos in pos[2:length(pos)]) {
				if (!is.null(infos[[ppos]])) {
					st = en + 1
					en = st + infos[[ppos]]@l - 1
					infos[[ppos]]@start = st
					infos[[ppos]]@end = en
					if (length(infos[[ppos]]@x)>2) minx = 0 # any allelic imbalance for complex CNAs
					else minx = 1 # only heterozygous option for simple CNAs
					status = as.vector(unlist(lapply(infos[[ppos]]@x, function(xx) sample(minx:ceiling(xx/2),1))))
					aPiece = piece(l=infos[[ppos]]@l,w=infos[[ppos]]@w,x=infos[[ppos]]@x,bfreq=status,loh=infos[[ppos]]@h,bn=noiseBAF,ln=noiseLRR,
						pfbs=pfb[st:en],constP=extremeBAF,redBafExtrs=reduceExtrBAF,lrrLevels=lrrLevels,constLrrNoise=constLRRNoise)
					baf = c(baf,aPiece@baf)
					lrr = c(lrr,aPiece@lrr)
				}
			}
		}
		
		dnaindex = 0
		for (info in infos)
			dnaindex = dnaindex + sum(info@x*info@w) * (info@l/totalLength)
		dnaindex = dnaindex / 2
		
		noise = rnorm(length(lrr))
		p = 200 # AR(p)
		maq = round(200 * (1+rnorm(1,sd=0.1))) # MA(q)
		nAR = lowess(noise, f= p / length(noise))$y
		maq = min(maq,totalLength/2) # otherwise, next line would produce NAs
		nARMA = kernapply(c(nAR,nAR[1:(2*maq)]),kernel("daniell",maq)) 
		nARMA = nARMA * waveSD / sd(nARMA)
		lrr = lrr+nARMA
		
		if (waveGCSD>0) {
			noise = rnorm(length(lrr))
			p = 2000 # AR(p)
			maq = round(2000 * (1+rnorm(1,sd=0.1))) # MA(q)
			nAR = lowess(noise, f= p / length(noise))$y
			maq = min(maq,totalLength/2) # otherwise, next line would produce NAs
			nARMA = kernapply(c(nAR,nAR[1:(2*maq)]),kernel("daniell",maq)) 
			nARMA = nARMA * waveGCSD / sd(nARMA)
			lrr = lrr+nARMA
		}
		
		baselevel = rnorm(1,mean=baseMean,sd=baseSD) # base intensity level
		if (!autoBase)
			shift = baselevel
		else
			shift = rnorm(1, mean = -0.46 * dnaindex + 0.46, sd = 0.058)
		lrr = lrr + shift
		
		regions = readOriginalCalls(infos)

		save(regions,shift,lrr,baf,dnaindex,purity,file=paste(annotationDir,"/",meta,"_metainfo.Rdata",sep=""))
		
		if (hybrid) chrs = rep(floor(scaffold$Chr), scaffold$Len)
		else chrs = rep(1,length(lrr))
		
		distSNP = round(distSNP)
		if (varDistSNP) {
			maxpos = (length(lrr)+1) * distSNP
			pos = sort(sample(c(1:maxpos), length(lrr)))
		} else {
			pos = seq(1, length(lrr)*distSNP, distSNP)
		}
		
		synth = matrix(ncol=5,nrow=0)
		synth = do.call(rbind,lapply(1:length(lrr), function(x) synth = c(synth, c(paste("snp",x,sep=""),chrs[x],pos[x],round(lrr[x],3),round(baf[x],3)))))
		colnames(synth) = c("Name","Chr","Position","XSynth.Log R Ratio","XSynth.B Allele Freq")
		write.table(synth,paste(lrrFirstDir,"/",meta,".lrrbaf.txt",sep=""),row.names=FALSE,sep="\t",quote=FALSE)
		
		synth = matrix(ncol=5,nrow=0)
		synth = do.call(rbind,lapply(1:length(lrr), function(x) synth = c(synth, c(paste("snp",x,sep=""),chrs[x],pos[x],round(baf[x],3),round(lrr[x],3)))))
		colnames(synth) = c("Name","Chr","Position","XSynth.B Allele Freq","XSynth.Log R Ratio")
		write.table(synth,paste(bafFirstDir,"/",meta,".baflrr.txt",sep=""),row.names=FALSE,sep="\t",quote=FALSE)

		if (isFirst) {
			sinfo = matrix(ncol=4,nrow=0)
			sinfo = do.call(rbind,lapply(1:length(lrr), function(x) c(paste("snp",x,sep=""),chrs[x],pos[x],NA)))
			sinfo[,4] = pfb
			colnames(sinfo) = c("Name","Chr","Position","PFB")
			write.table(sinfo,paste(annotationDir,"/","pfb",sep=""),row.names=FALSE,sep="\t",quote=FALSE)
			sinfo = do.call(rbind,lapply(1:length(lrr), function(x) c(paste("snp",x,sep=""),chrs[x],pos[x],40)))
			colnames(sinfo) = c("Name","Chr","Position","GC")
			write.table(sinfo,paste(annotationDir,"/","gcmodel",sep=""),row.names=FALSE,sep="\t",quote=FALSE)
			isFirst = FALSE
		}
		
		cat("Done.\n")
	}
}
