\name{cnagen}
\Rdversion{2.1}
\alias{cnagen}

\title{
Generation of synthetic data to emulate tumour samples on SNP arrays
}
\description{
Generation of synthetic SNP-array tumour samples with extensive parameterization. Normal cell contamination, intra-tumour heterogeneity, genomic waves, baseline shift and other known factors are parameterizable.
}
\usage{
cnagen(lengths = lapply(1:6, function(x) c(100)), times = rep(1, 6), weights2 = c(0.66), weights3 = c(0.66), copynumbers = list(list(c(1)), list(c(1, 3)), list(c(1, 3, 4))), replicates = 1, purities = c(1, 0.5), distSNP = 1, varDistSNP = FALSE, noiseBAF = 0.03, extremeBAF = 0.3, reduceExtrBAF = TRUE, pfbMode = "array", noiseLRR = 0.2, constLRRNoise = TRUE, lrrLevels = c(-0.75, -0.5, 0, 0.29, 0.5, 0.66, 0.79, 0.9, 1), autoBase = TRUE, baseMean = 0, baseSD = 0.1, waveSD = 0.02, annotationDir = "data", lrrFirstDir = "data_lrrbaf", bafFirstDir = "data_baflrr", scaffold = NULL, waveGCSD = 0.03, intratHet = 0)
}

\arguments{
  \item{lengths}{
The set of lengths for the 6 region types (normal, simple CNA, somatic LOH, germline LOH, 2-subclone CNA, 3-subclone CNA). Use a list of vectors.
}
  \item{times}{
For each region type, number of times that each unique region is included.
}
  \item{weights2}{
Proportions that the main subclone will account for in 2-subclone CNAs.
}
  \item{weights3}{
Proportions that the main subclone will account for in 3-subclone CNAs.
}
  \item{copynumbers}{
Copy numbers for simple (normal and both LOH) regions, and vectors of copy numbers for 2-subclone and 3-subclone regions. 
}
  \item{replicates}{
How many times to generate samples at each of the specified purities.
}
  \item{purities}{
Tumour purities at which to generate samples. Purity is the opposite of normal cell contamination.
}
  \item{distSNP}{
Distance at which the SNPs are generated.
}
  \item{varDistSNP}{
Whether to make distances between SNPs variable. If true, the distances will follow a distribution that approximates Poisson.
}
  \item{noiseBAF}{
Noise factor for the BAF signal. Default standard deviation is 0.03 (typical in Illumina arrays).
}
  \item{extremeBAF}{
Proportion of homozygous SNPs forced to take either 0 or 1 values in the BAF signal.
}
  \item{reduceExtrBAF}{
Whether to reduce noise in the homozygous bands of the BAF signal.
}
  \item{pfbMode}{
Select a PFB sampling mode from between 'array','threepeaks','uniform', 'maxinfo' and a value between zero and one.
}
  \item{noiseLRR}{
Noise factor for the LRR signal. If left to default and constant standard deviation is set to TRUE, this will be 0.20 (typical in Illumina arrays).
}
  \item{constLRRNoise}{
Whether to use the same standard deviation for all copy numbers in the LRR signal. If not, it decreases with copy number increase.
}
  \item{lrrLevels}{
Expected LRR levels for the different copy numbers, starting with copy number 0.
}
  \item{autoBase}{
Overrides baseMean and baseSD. Calculates LRR shift and SD from mean shift based on DNA index and their empirically observed correlation on a set of samples genotyped with 300k Illumina SNP arrays (Human Hap300-Duo).
}
  \item{baseMean}{
How much to shift the LRR levels. Can be positive or negative.
}
  \item{baseSD}{
Standard deviation for the variability in the LRR shift.
}
  \item{waveSD}{
Standard deviation of the autocorrelated bias due to genomic waves, as described in Diskin et al., 2008. Default is 0.02.
}
  \item{annotationDir}{
Output directory for the GC model and PFB files, and also additional R data that includes diploid genotype and region information. 
}
  \item{lrrFirstDir}{
Output directory for the samples in which the LRR column goes before the BAF column. Use them with OncoSNP.
}
  \item{bafFirstDir}{
Output directory for the samples in which the BAF column goes before the LRR column.
}
  \item{scaffold}{
A data frame with 5 columns: chromosome, length, presence of germline LOH, copy number, B allele count. Each row represents a genomic region. Rows should be ordered by genomic position.
}
  \item{waveGCSD}{
Additional long-range genomic waves, such as those resulting from GC content variability. The parameter represents the standard deviation of the noise.
}
  \item{intratHet}{
A value from 0 to 1 that defines the maximum amount of intra-tumour heterogeneity within each region.
}
}
\details{
The method generates synthetic data with up to 6 different region types (normal, simple CNA, somatic LOH, germline LOH, 2-subclone CNA, 3-subclone CNA). This is the reason that lengths is comprised by 6 vectors and, times, by 6 numbers. Each vector in lengths specifies the set of lengths that regions of each type will take. For instance, if the first vector is c(100,1000), there will be regions with normal genotype of lengths 100 and 1000 SNPs. In turn, a region with a certain length, copy number and of certain type will appear once unless the corresponding number in times is higher than 1. In the case of complex CNAs, the corresponding number in times reflects the times a CNA with a certain length, copy number distribution and main subclone proportion appears. 

copynumber is expected to be a list of lists, where the first list contains vectors of length one; the second list, vectors of length 2; and the third list, vectors of length 3.

PFB sampling mode 'array' samples from B allele frequency of SNPs in an Illumina Human660W-Quad array; 'uniform', from a uniform distribution; 'maxinfo' maximizes genotype information with a constant PFB of 0.5; 'threpeaks' samples values from 0, 0.5 and 1, approximating PFB distribution in the human genome.
}
\value{

\item{pfb}{A file with the PFBs, sampled in the selected mode, used in the generation.}
\item{gcmodel}{A file with dummy GC content values. For use with methods that require such input (e.g. OncoSNP, GPHMM).}

\item{Rdata files}{A file with R objects for diploid genotype, region information, LRR shift, and BAF and LRR signals.}
\item{Intensity files}{A tab-delimited file with 5 columns (SNP name, chromosome and position, BAF value and LRR value).}
\item{Alternative intensity files}{A tab-delimited file with 5 columns (SNP name, chromosome and position, LRR value and BAF value).}
}
\references{
%% ~put references to the literature/web site here ~
}
\author{
David Mosen-Ansorena <dmosen.gn@cicbiogune.es>
}
\note{
%%  ~~further notes~~
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
}
\examples{
lengths = lapply(1:6, function(x) c(10,20,40,80,160)) # there are regions from 10 to 160 SNPs regardless of the region type
times = c(rep(5,2),rep(1,4)) # each unique normal region and simple non-LOH CNA appears 5 times

x1 = list(c(1), c(2), c(3), c(4), c(5)) # copy numbers up to 5 for simple regions
x2 = list(c(1,3), c(3,4)) # loss-gain and gain-gain combinations for 2-subclone CNAs
x3 = list(c(1,3,4), c(3,4,5)) # loss-gain-gain and three gain combinations for 3-subclone CNAs

cnagen(lengths = lengths, times = times, weights3 = c(0.5,0.7), copynumbers = list(x1,x2,x3), # CNA parameters
		replicates = 1000, purities = c(0.5), # sample parameters
		noiseBAF = 1.2, extremeBAF = 0.3, reduceExtrBAF = TRUE, pfbMode = "array", # BAF parameters
		noiseLRR = 1.1, constLRRNoise = TRUE, lrrLevels = c(-.75,-.5,0,.29,.5,.66,.79,.9,1), baseMean = -0.012, baseSD = 0.05, waveSD = 0.02, # LRR parametrs
		annotationDir = "annot", lrrFirstDir = "samples_o", bafFirstDir = "samples") # output parameters
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
